
#include <system.h>
#include "logger.h"
#include "piccode.h"

#pragma CLOCK_FREQ 8000000
#pragma RS232_BAUD 9600

asm {
list p=16F628
__config H'3F01'
}

// Global variables declaration
short     timer1_clone@0x0E;

char time[6]; //time[5]=year,...,time[0]=sec
char cmd,setcount;
int eventcount;
char newinput,oldinput;
char ready;


////////////////////////////////////

void on_init( void )
	{
     option_reg = 0x7F;
     trisa = 0x3F;
     porta = 0x00;
     trisb = 0xFF;
     portb = 0x00;

	 vrcon=0;
	 cmcon=7;

     //timer1
     t1con = 0x0F;

     // rs232 communications setup
//     spbrg = 12;//25  txsta = 0x26; & 4mhz
//     txsta = 0x22;
     spbrg = 12;
     txsta = 0x26;

     rcsta = 0x90;

//25  txsta = 0x26; & 4mhz 9600
//12  txsta = 0x26; & 4mhz 19200

     set_bit( pie1, RCIE );		//USART RX
     set_bit( pie1, TMR1IE );	//TMR1
  
     enable_interrupt( RBIE );	//RB
     enable_interrupt( PEIE );	//PERIPHERAL
     enable_interrupt( GIE );	//GLOBAL
	}

void incdate()
     {
     char maxday=30;
     switch (time[4])
          {
          case 1:
          case 3:
          case 5:
          case 7:
          case 8:
          case 10:
          case 12:maxday=31;break;
          case 2:{if (time[5]%4==0) maxday=29;else maxday=28;};break;
          }
     if (time[3]==maxday)
          {
          time[3]=1;
          if (time[4]==12)
               {
               time[4]=1;
               time[5]=time[5]+1;
               }
          else time[4]=time[4]+1;
          }
     else time[3]=time[3]+1;
     }

void inctime()
     {
     if (time[0]==59)
          {
          time[0]=0;
          if (time[1]==59)
               {
               time[1]=0;
               if (time[2]==23)
                    {
                    time[2]=0;
                    incdate();
                    }
               else time[2]=time[2]+1;
               }
          else time[1]=time[1]+1;
          }
     else time[0]=time[0]+1;
     }

void writeflash()
	{
	if (eventcount>4095)  return;
	int addr=eventcount<<3;
	
	i2c_open(0xA0);
	i2c_send_byte(addr>>8);
	i2c_send_byte(addr & 0xff);
	char k;
	for (k=0;k<6;k++)
		 i2c_send_byte(time[k]);
	i2c_send_byte(newinput);
	i2c_send_byte(0x55);
	i2c_stop();
	}

void readflash(int pos)
	{
	int addr=pos<<3;
	i2c_open(0xA0);
	i2c_send_byte(addr>>8);
	i2c_send_byte(addr & 0xff);
	i2c_open(0xA1);
	char l;
	for (l=0;l<8;l++)
		 {
		 txreg_putch(i2c_read_byte());
		 i2c_send_acknowledge(l<7);
		 }
	i2c_stop();
	}


void interrupt( void )
	{

	oldinput=newinput;
	newinput=((portb>>2)&0x0E)|(portb&1);
	if ((oldinput!=newinput) & ready)
	   {
	   writeflash();
	   eventcount++;
	   }
    ready=1;

	 if (intcon&0x01)
		{
		clear_bit( intcon, 0);
		}
	 
     if ( pie1&0x01 && pir1&0x01 ) //time[1]R1
          {
          tmr1h=0x80;
          inctime();
          clear_bit( pir1, TMR1IF );
          }

     if ( pie1&0x10 && pir1&0x10 ) // TXREG has been sent out
          {
          clear_bit( pie1, TXIE );
          }

     if ( pir1&0x20 && pie1&0x20 ) // byte received interrupt
		 {
		 if (setcount==0)
			  cmd=rcreg;
		 else
			  {
			  time[setcount-1]=rcreg;
			  setcount++;
			  if (setcount>6) setcount=0;
			  }
		 }
}

main()
     {
     time[5]=0;//year
     time[4]=1;//month
     time[3]=1;//date
     time[2]=0;time[1]=0;time[0]=0;
     cmd=0;
     setcount=0;
     eventcount=0;
     ready=1;

     on_init();

     for (;;)
		 {
			
		 if (cmd>0)
			  {
			  if (cmd==0x10)
				   {
				   //get time ss mm hh dd mm yy
				   char i;
				   for (i=0;i<6;i++)
						txreg_putch(time[i]);
				   }
	
			  if (cmd==0x11)
				   {
				   //set time
				   setcount=1;
				   }
	
			  if (cmd==0x20)
				   {
				   //clear history
				   eventcount=0;
				   }
			  if (cmd==0x21)//'!'
				   {
				   //get history size
				   txreg_putch(eventcount>>8);
				   txreg_putch(eventcount & 0xFF);
				   }
			  if (cmd==0x22)//'"'
				   {
				   //get history
				   char j;
				   for (j=0;j<eventcount;j++)
						readflash(j);
				   }
	
			  if (cmd==0x54)//'T'
				   {
				   txreg_putch('O');
				   txreg_putch('K');
				   }
			  cmd=0;
			  }

		 if (porta & 4) //PC connected
			{
			set_bit(rcsta, SPEN);
			}
		else 
			{
			clear_bit(rcsta, SPEN);
			sleep();				
			}
		 }

     }
	